#include "auto.h"

/************************************************************************/
/* 				GLOBALS					*/
/************************************************************************/
double x_step, y_step;
int rows, cols;
int paint_all = 1;
stack state_list[MAX_COLORS + 1];

/************************************************************************/
/*			PRIVATE FUNCTIONS				*/
/************************************************************************/
void PaintAutomataWidget (SUIT_object obj);
void HitAutomataWidget (SUIT_object obj, SUIT_event ev);
void PaintSquare (automaton automata, int row, int col, int item, int grid);
void PaintAutomatonGrid (SUIT_object auto_obj);
void PaintAutomatonGridRowCol (SUIT_object auto_obj, int row, int col);

void init_state_list ();
void paint_state_list (automaton automata, int grid);


/************************************************************************/
/* User defined  SUIT objects						*/
/************************************************************************/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	CreateAutomataWidget				*/
/* 									*/
/* Description:		create a new instance of the graphics widget	*/
/*			for displaying Automata-nets			*/
/* 									*/
/* Date of creation:	16-10-92					*/
/* 									*/
/* input arguments:	name of widget					*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	pointer to new SUIT object			*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

SUIT_object
CreateAutomataWidget (char *name)
{
  SUIT_object obj;
  SUIT_object board, rows_label, cols_label, states_label;
  SUIT_object rows_obj, cols_obj, states_obj;
  SUIT_object step_obj, grid_obj;
  SUIT_object clear_obj, refresh_obj, file_obj, color_obj;
  SUIT_object save_obj, load_obj, save_pgm_obj, load_pgm_obj;
  SUIT_object method_obj;
  SUIT_textList method_list;
  rule *current_rule;

  int states;
  automaton automata;
  char buffer[BUFFER_SIZE];
  char *obj_name;


  rows = DEFAULT_ROWS;
  cols = DEFAULT_COLS;
  states = DEFAULT_STATES;

  /* ----------------------------------------------------------------	*/
  /* 				create the bulletin board		*/
  /* ----------------------------------------------------------------	*/
  board = SUIT_createBulletinBoard (name);


  /* ----------------------------------------------------------------	*/
  /* 		basic setup of automaton object				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "automata");

  obj = SUIT_createObject (obj_name, "Automata class");
  SUIT_addDisplayToObject
    (
      obj,
      "standard",
      HitAutomataWidget,
      PaintAutomataWidget
    );

  /* ----------------------------------------------------------------	*/
  /* allocate space for cellular automaton				*/
  /* ----------------------------------------------------------------	*/
  automata = AllocateAutomaton (rows, cols);

  /* ----------------------------------------------------------------	*/
  /*			set automaton properties			*/
  /* ----------------------------------------------------------------	*/
  SUIT_setInteger (obj, ROWS, rows);
  SUIT_setInteger (obj, COLUMNS, cols);
  SUIT_setInteger (obj, STATES, states);
  SUIT_setBoolean (obj, GRID, FALSE);
  SUIT_setBoolean (obj, HAS_BACKGROUND, FALSE);
  SUITx_setVoidPtr (obj, AUTOMATON, automata);
  SUITx_setVoidPtr (obj, UPDATE_STACK, NULL);
  SUITx_setVoidPtr (obj, DISPLAY_STACK, NULL);

  SUIT_makePropertyTemporary (obj, ROWS, OBJECT);
  SUIT_makePropertyTemporary (obj, COLUMNS, OBJECT);
  SUIT_makePropertyTemporary (obj, STATES, OBJECT);
  SUIT_makePropertyTemporary (obj, GRID, OBJECT);
  SUIT_makePropertyTemporary (obj, AUTOMATON, OBJECT);
  SUIT_makePropertyTemporary (obj, UPDATE_STACK, OBJECT);
  SUIT_makePropertyTemporary (obj, DISPLAY_STACK, OBJECT);

  SUIT_lockProperty (obj, ROWS, OBJECT);
  SUIT_lockProperty (obj, COLUMNS, OBJECT);
  SUIT_lockProperty (obj, STATES, OBJECT);
  SUIT_lockProperty (obj, GRID, OBJECT);
  SUIT_lockProperty (obj, AUTOMATON, OBJECT);
  SUIT_lockProperty (obj, UPDATE_STACK, OBJECT);
  SUIT_lockProperty (obj, DISPLAY_STACK, OBJECT);

  /* ----------------------------------------------------------------	*/
  /* 			add automata to the bulletin board		*/
  /* ----------------------------------------------------------------	*/
  SUIT_setObject (board, AUTOMATON, obj);
  SUIT_makePropertyTemporary (board, AUTOMATON, OBJECT);

  /* ----------------------------------------------------------------	*/
  /* 			create the labels				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "rlabel");
  rows_label = SUIT_createLabel (obj_name);
  SUIT_setText (rows_label, LABEL, "Rows");

  obj_name = SUIT_relativeName (board, "clabel");
  cols_label = SUIT_createLabel (obj_name);
  SUIT_setText (cols_label, LABEL, "Cols");

  obj_name = SUIT_relativeName (board, "slabel");
  states_label = SUIT_createLabel (obj_name);
  SUIT_setText (states_label, LABEL, "States");

  /* ----------------------------------------------------------------	*/
  /*			create type in boxes				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "stype");
  states_obj = SUIT_createTypeInBox (obj_name, adjust_states);
  SUIT_makePropertyTemporary (states_obj, CURRENT_VALUE, OBJECT);

  sprintf (buffer, "%d", states);
  SUIT_setText (states_obj, CURRENT_VALUE, buffer);

  /*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

  obj_name = SUIT_relativeName (board, "rtype");
  rows_obj = SUIT_createTypeInBox (obj_name, adjust_rows);
  SUIT_makePropertyTemporary (rows_obj, CURRENT_VALUE, OBJECT);

  sprintf (buffer, "%d", rows);
  SUIT_setText (rows_obj, CURRENT_VALUE, buffer);

  /*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

  obj_name = SUIT_relativeName (board, "ctype");
  cols_obj = SUIT_createTypeInBox (obj_name, adjust_cols);
  SUIT_makePropertyTemporary (cols_obj, CURRENT_VALUE, OBJECT);

  sprintf (buffer, "%d", cols);
  SUIT_setText (cols_obj, CURRENT_VALUE, buffer);

  /* ----------------------------------------------------------------	*/
  /*                    add a  toggle switches				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "grid");
  grid_obj = SUIT_createOnOffSwitch (obj_name, change_grid);
  SUIT_setBoolean (grid_obj, SHRINK_TO_FIT, FALSE);
  SUIT_setText (grid_obj, LABEL, "Grid");
  SUIT_setEnumString (grid_obj, BORDER_TYPE, "fancy motif");

  SUIT_setBoolean (grid_obj, CURRENT_VALUE, FALSE);
  SUIT_makePropertyTemporary (grid_obj, CURRENT_VALUE, OBJECT);



  obj_name = SUIT_relativeName (board, "step");
  step_obj = SUIT_createOnOffSwitch (obj_name, change_step);
  SUIT_setBoolean (step_obj, SHRINK_TO_FIT, FALSE);
  SUIT_setText (step_obj, LABEL, "Step");
  SUIT_setEnumString (step_obj, BORDER_TYPE, "fancy motif");

  SUIT_setBoolean (step_obj, CURRENT_VALUE, FALSE);
  SUIT_makePropertyTemporary (step_obj, CURRENT_VALUE, OBJECT);




  /* ----------------------------------------------------------------	*/
  /*			add save and load buttons			*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "menu");
  file_obj = SUIT_createPullDownMenu (obj_name);
  SUIT_setText (file_obj, LABEL, "File..");
  SUIT_setBoolean (file_obj, SHRINK_TO_FIT, FALSE);
  SUIT_setEnumString (file_obj, BORDER_TYPE, "fancy motif");

  obj_name = SUIT_relativeName (file_obj, "load_auto");
  load_obj = SUIT_addToMenu (file_obj, obj_name, load_auto);
  SUIT_setText (load_obj, LABEL, "LOAD ...");
  SUIT_setEnumString (load_obj, BORDER_TYPE, "fancy motif");
  SUIT_setColor (load_obj, BACKGROUND_COLOR, GP_defColor ("blue"));
  SUIT_setColor (load_obj, FOREGROUND_COLOR, GP_defColor ("white"));

  obj_name = SUIT_relativeName (file_obj, "load_pgm_auto");
  load_pgm_obj = SUIT_addToMenu (file_obj, obj_name, load_pgm_auto);
  SUIT_setText (load_pgm_obj, LABEL, "LOAD @b((PGM)) ...");
  SUIT_setEnumString (load_pgm_obj, BORDER_TYPE, "fancy motif");

  obj_name = SUIT_relativeName (file_obj, "save_auto");
  save_obj = SUIT_addToMenu (file_obj, obj_name, save_auto);
  SUIT_setText (save_obj, LABEL, "SAVE ...");
  SUIT_setEnumString (save_obj, BORDER_TYPE, "fancy motif");
  SUIT_setColor (save_obj, BACKGROUND_COLOR, GP_defColor ("blue"));
  SUIT_setColor (save_obj, FOREGROUND_COLOR, GP_defColor ("white"));

  obj_name = SUIT_relativeName (file_obj, "save_pgm_auto");
  save_pgm_obj = SUIT_addToMenu (file_obj, obj_name, save_pgm_auto);
  SUIT_setText (save_pgm_obj, LABEL, "SAVE @b((PGM)) ...");
  SUIT_setEnumString (save_pgm_obj, BORDER_TYPE, "fancy motif");

  /* ----------------------------------------------------------------	*/
  /*			add a clear button				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "clear");
  clear_obj = SUIT_createButton (obj_name, clear_auto);
  SUIT_setText (clear_obj, LABEL, "Clear");
  SUIT_setBoolean (clear_obj, SHRINK_TO_FIT, FALSE);
  SUIT_setEnumString (clear_obj, BORDER_TYPE, "fancy motif");

  /* ----------------------------------------------------------------	*/
  /*			add a refresh button				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "refresh");
  refresh_obj = SUIT_createButton (obj_name, refresh_auto);
  SUIT_setText (refresh_obj, LABEL, "Refresh");
  SUIT_setBoolean (refresh_obj, SHRINK_TO_FIT, FALSE);
  SUIT_setEnumString (refresh_obj, BORDER_TYPE, "fancy motif");

  /* ----------------------------------------------------------------	*/
  /*   		 	add a colorbar widget				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "colorbar");
  color_obj = CreateColorbarWidget (obj_name);
  SUIT_setInteger (color_obj, CSTATES, states);
  SUIT_makePropertyTemporary (color_obj, CSTATES, OBJECT);
  SUIT_setEnumString (color_obj, ACTIVE_DISPLAY, "standard");

  /* ----------------------------------------------------------------	*/
  /* 			add a method widget				*/
  /* ----------------------------------------------------------------	*/
  obj_name = SUIT_relativeName (board, "method");
  method_obj = SUIT_createScrollableList (obj_name, do_method);

  method_list = SUIT_getTextList (method_obj, "LIST");

  /*
     spaces are placed infront of the names as there seems to be a bug in
     SUIT with bulletin boards . remember to trim the name when using
  */

  current_rule = rule_array;
  while (current_rule->name)
  {
    char buffer[30];

    sprintf (buffer, "    %s", current_rule->name);
    printf ("Adding method %s\n", buffer);

    SUIT_appendToTextList (method_list, buffer);
    current_rule++;
  }

  SUIT_setTextList (method_obj, "LIST", method_list);

  SUIT_setBoolean (method_obj, HAS_BACKGROUND, TRUE);
  SUIT_setText (method_obj, LABEL, "Rules");
  SUIT_makePropertyTemporary (method_obj, LIST, OBJECT);

  /* ---------------------------------------------------------------- */
  /* 		keep a track of type-in, colorbar object              */
  /* ---------------------------------------------------------------- */
  SUIT_setObject (obj, ROWSOBJ, rows_obj);
  SUIT_makePropertyTemporary (obj, ROWSOBJ, OBJECT);

  SUIT_setObject (obj, STATEOBJ, states_obj);
  SUIT_makePropertyTemporary (obj, STATEOBJ, OBJECT);

  SUIT_setObject (obj, COLSOBJ, cols_obj);
  SUIT_makePropertyTemporary (obj, COLSOBJ, OBJECT);

  SUIT_setObject (states_obj, COLOROBJ, color_obj);
  SUIT_makePropertyTemporary (states_obj, COLOROBJ, OBJECT);


  /* ----------------------------------------------------------------	*/
  /* 			stick objects together				*/
  /* ----------------------------------------------------------------	*/
  SUIT_addChildToObject (board, obj);
  SUIT_addChildToObject (board, rows_label);
  SUIT_addChildToObject (board, cols_label);
  SUIT_addChildToObject (board, rows_obj);
  SUIT_addChildToObject (board, states_label);
  SUIT_addChildToObject (board, cols_obj);
  SUIT_addChildToObject (board, states_obj);
  SUIT_addChildToObject (board, grid_obj);
  SUIT_addChildToObject (board, step_obj);
  SUIT_addChildToObject (board, file_obj);
  SUIT_addChildToObject (board, clear_obj);
  SUIT_addChildToObject (board, refresh_obj);
  SUIT_addChildToObject (board, color_obj);
  SUIT_addChildToObject (board, method_obj);

  /* ----------------------------------------------------------------	*/
  /*  		position the objects				*/
  /* ----------------------------------------------------------------	*/

  setchildsize (obj, 0.15, 0.11, 1.0, 1.0);

  setchildsize (step_obj, 0.0, 0.96, 0.14, 1.00);
  setchildsize (grid_obj, 0.0, 0.90, 0.14, 0.94);

  setchildsize (file_obj, 0.0, 0.81, 0.14, 0.85);
  setchildsize (clear_obj, 0.0, 0.75, 0.14, 0.79);
  setchildsize (refresh_obj, 0.0, 0.69, 0.14, 0.73);

  setchildsize (method_obj, 0.0, 0.11, 0.14, 0.65);

#define BOX_BOTTOM 0.00
#define BOX_TOP    0.075

  setchildsize (rows_label, 0.0, BOX_BOTTOM, 0.04, BOX_TOP);
  setchildsize (rows_obj, 0.07, BOX_BOTTOM, 0.13, BOX_TOP);

  setchildsize (cols_label, 0.17, BOX_BOTTOM, 0.20, BOX_TOP);
  setchildsize (cols_obj, 0.22, BOX_BOTTOM, 0.27, BOX_TOP);

  setchildsize (states_label, 0.34, BOX_BOTTOM, 0.38, BOX_TOP);
  setchildsize (states_obj, 0.40, BOX_BOTTOM, 0.45, BOX_TOP);

  setchildsize (color_obj, 0.52, BOX_BOTTOM, 1.0, BOX_TOP);

  return (board);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:	HitAutomataWidget				*/
/* 									*/
/* Description:		This determines what happens when a mouse event */
/*			on the graphics window				*/
/* 									*/
/* Date of creation:	29-01-93					*/
/* 									*/
/* input arguments:	obj		the object which was hit	*/
/*			ev		the mouse event			*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	void						*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void
  HitAutomataWidget (SUIT_object obj, SUIT_event ev)
{
  int grid;
  double x, y;
  int row, col, states, item;
  automaton automata;

  /* ----------------------------------------------------------------	*/
  /*		always update the x/y/coordinates			*/
  /* ----------------------------------------------------------------	*/

  x = ev.worldLocation.x;
  y = ev.worldLocation.y;
  rows = SUIT_getInteger (obj, ROWS);
  cols = SUIT_getInteger (obj, COLUMNS);
  states = SUIT_getInteger (obj, STATES);
  grid = SUIT_getBoolean (obj, GRID);
  automata = (automaton) SUITx_getVoidPtr (obj, AUTOMATON);

  /* ----------------------------------------------------------------	*/
  /* which row/col was hit						*/
  /* ----------------------------------------------------------------	*/
  row = floor (y * rows);
  col = floor (x * cols);

  if (col < 0)
    col = 0;
  if (col >= cols)
    col = cols - 1;

  if (row < 0)
    row = 0;
  if (row >= rows)
    row = rows - 1;

  item = get_item (row, col);
  y_step = 1.0 / (double) rows;
  x_step = 1.0 / (double) cols;

  /* ----------------------------------------------------------------	*/
  /* process the event							*/
  /* ----------------------------------------------------------------	*/

  switch (ev.type)
  {
  case CLICK:
  case MOUSE_DOWN:
  case MOUSE_MOTION:
  case MOUSE_UP:
    if (automata[item].state < states)
    {
      automata[item].state++;
      automata[item].changed = TRUE;
    }
    set_gray (automata[item].state);
    PaintSquare (automata, row, col, item, grid);
    PaintAutomatonGridRowCol (obj, row, col);
    break;

  case KEYSTROKE:
    if (automata[item].state > 0)
    {
      automata[item].state--;
      automata[item].changed = TRUE;
    }
    set_gray (automata[item].state);
    PaintSquare (automata, row, col, item, grid);
    PaintAutomatonGridRowCol (obj, row, col);
    break;

  default:
    break;
  }

  /* ----------------------------------------------------------------	*/
  /* the stacks have been invalidated, they should be trashed		*/
  /* ----------------------------------------------------------------	*/
  ClearAutomatonStacks (obj);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:	PaintSquare					*/
/*									*/
/* Description:		paints automata element, given row,col,item.	*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/*	item can be trivially calculated, but as it has allready been	*/
/*	done before we got here, why slow things by doing it again?	*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void PaintSquare
  (automaton automata, int row, int col, int item, int grid)
{
  int state, changed;
  GP_point bottom_left, top_right;


  state = automata[item].state;
  changed = automata[item].changed;

  if (changed || (paint_all && (state != 0)))
  {

    bottom_left.x = (double) col *x_step;
    bottom_left.y = (double) row *y_step;

    top_right.x = bottom_left.x + x_step;
    top_right.y = bottom_left.y + y_step;

    GP_fillRectanglePt (bottom_left, top_right);

    automata[item].changed = FALSE;
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	PaintAutomataWidget				*/
/* 									*/
/* Description:		draw the contents of widget			*/
/* 									*/
/* Date of creation:	16-10-92					*/
/* 									*/
/* input arguments:	obj	object to redraw			*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	void						*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void
  PaintAutomataWidget (SUIT_object obj)
{
  int row, col, grid, item, state;
  automaton automata;
  stack display_stack, update_stack;


  ENTER ("PaintAutomataWidget");

  rows = SUIT_getInteger (obj, ROWS);
  cols = SUIT_getInteger (obj, COLUMNS);
  grid = SUIT_getBoolean (obj, GRID);
  automata = SUITx_getVoidPtr (obj, AUTOMATON);

  display_stack = (stack) SUITx_getVoidPtr (obj, DISPLAY_STACK);
  update_stack = (stack) SUITx_getVoidPtr (obj, UPDATE_STACK);

  /* ------------------------------------------------------------------	*/
  /* initialise these two everytime the widget is painted		*/
  /* ------------------------------------------------------------------	*/
  y_step = 1.0 / (double) rows;
  x_step = 1.0 / (double) cols;

  /* ------------------------------------------------------------------	*/
  /*		FIRST PASS ... create list to paint			*/
  /* approach depends on whether there was a display_stack		*/
  /* ------------------------------------------------------------------	*/
  init_state_list ();

  if ((display_stack) || (update_stack))
  {
    if (update_stack)
    {
      while (update_stack)
      {
	item = update_stack->data;
	state = automata[item].state;
	stack_push (&state_list[state], item);
	update_stack = stack_next (update_stack);
      }
      stack_destroy (&update_stack);
      SUITx_setVoidPtr (obj, UPDATE_STACK, NULL);
    }
    else
      while (display_stack)
      {
	item = display_stack->data;
	state = automata[item].state;
	stack_push (&state_list[state], item);
	display_stack = stack_next (display_stack);
      }
  }
  else
  {
    /* ----------------------------------------------------------------	*/
    /* The Brute force approach						*/
    /* ----------------------------------------------------------------	*/

    item = 0;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	state = automata[item].state;
	stack_push (&state_list[state], item);
	item++;
      }
  }

  /* ------------------------------------------------------------------	*/
  /* SECOND PASS display the contents of the colour list		*/
  /* ------------------------------------------------------------------	*/
  paint_state_list (automata, grid);

  /* ------------------------------------------------------------------	*/
  /* ------------------------------------------------------------------	*/
  PaintAutomatonGrid (obj);

  LEAVE ("PaintAutomataWidget");
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void
  InitAutomaton (automaton automata)
{
  int row, col, item;

  if (automata)
  {
    item = 0;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	automata[item].timer = 0;
	automata[item].state = 0;
	automata[item].tmp = 0;
	automata[item].changed = TRUE;
	automata[item].checked = FALSE;
	automata[item].cumulative = 0;
	automata[item].cumulative_changed = TRUE;
	automata[item].unchanged = 0;
	item++;
      }

    resetPerfmeter (fired_obj);
    resetPerfmeter (active_obj);
    resetMatrix (core_obj);
  }
  else
    SUIT_inform ("attempt to initialise void pointer");


}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void
  PaintAutomatonGrid (SUIT_object auto_obj)
{
  int row, col;
  int grid;
  double x, y;

  grid = SUIT_getBoolean (auto_obj, GRID);

  if (grid)
  {
    set_colour ("yellow");

    y = y_step;
    for (row = 0; row < rows; row++)
    {
      GP_lineCoord (0.0, y, 1.0, y);
      y += y_step;
    }

    x = x_step;
    for (col = 0; col < cols; col++)
    {
      GP_lineCoord (x, 0.0, x, 1.0);
      x += x_step;
    }
  }
}

void PaintAutomatonGridRowCol (SUIT_object auto_obj, int row, int col)
{
  int grid;
  GP_point bottom_left, top_right;

  grid = SUIT_getBoolean (auto_obj, GRID);

  if (grid)
  {
    set_colour ("yellow");

    bottom_left.y = y_step * row;
    bottom_left.x = x_step * col;
    top_right.x = bottom_left.x + x_step;
    top_right.y = bottom_left.y + y_step;

    GP_rectanglePt (bottom_left, top_right);

  }
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:	allocate_automata				*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
automaton AllocateAutomaton (int nrows, int ncols)
{
  automaton automata;

  rows = nrows;
  cols = ncols;
  automata = (automaton) malloc ((nrows * ncols) * sizeof (element));

  InitAutomaton (automata);

  return (automata);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void ClearAutomatonStacks (SUIT_object obj)
{
  stack display_stack, update_stack;

  display_stack = SUITx_getVoidPtr (obj, DISPLAY_STACK);
  update_stack = SUITx_getVoidPtr (obj, UPDATE_STACK);

  stack_destroy (&display_stack);
  stack_destroy (&update_stack);

  SUITx_setVoidPtr (obj, DISPLAY_STACK, NULL);
  SUITx_setVoidPtr (obj, UPDATE_STACK, NULL);
}



/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:
*
* Description:
*
* Date of creation:
*
* input arguments:
*
* output:
*
* function return:
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void init_state_list ()
{
  int item;

  for (item = 0; item <= MAX_COLORS; item++)
    if (state_list[item])
      stack_destroy (&(state_list[item]));
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:
*
* Description:
*
* Date of creation:
*
* input arguments:
*
* output:
*
* function return:
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void paint_state_list (automaton automata, int grid)
{
  stack disp_stack;
  int row, col, colour;

  for (colour = 0; colour <= MAX_COLORS; colour++)
  {
    disp_stack = state_list[colour];
    if (disp_stack)
    {
      set_gray (colour);
      while (disp_stack)
      {
	get_rowCol (disp_stack->data, &row, &col);
	DEBUG_MSG( ("(%d -> %d, %d)\t", disp_stack->data, row, col));

	PaintSquare (automata, row, col, disp_stack->data, grid);
	disp_stack = stack_next (disp_stack);
      }
    }
  }
}
