#include "auto.h"

/************************************************************************/
/*			PRIVATE FUNCTIONS				*/
/************************************************************************/
void PaintColorbarWidget (SUIT_object obj);
void PaintColorbarWidget2 (SUIT_object obj);
void colorswap (SUIT_object obj, SUIT_event ev);


/************************************************************************/
/* User defined  SUIT objects						*/
/************************************************************************/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Function name: 	CreateColorbarWidget

   Description:		create a new instance of the graphics widget
			for displaying Colorbar-nets

   Date of creation:	16-10-92

   input arguments:	name of widget

   output:		None

   function return:	pointer to new SUIT object

   Modifications:

 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

SUIT_object
CreateColorbarWidget (char *name)
{
  int states;
  SUIT_object obj;

  states = DEFAULT_STATES;

  /* ------------------------------------------------------------------	*/
  /* 		basic setup of Colorbar object				*/
  /* ------------------------------------------------------------------	*/
  obj = SUIT_createObject (name, "Colorbar class");
  SUIT_addDisplayToObject
    (
      obj,
      "standard",
      NULL,
      PaintColorbarWidget
    );

  SUIT_addDisplayToObject
    (
      obj,
      "thermometer",
      NULL,
      PaintColorbarWidget2
    );


  SUIT_setInteger (obj, CSTATES, states);
  SUIT_makePropertyTemporary (obj, CSTATES, OBJECT);

  /* ------------------------------------------------------------------ */
  /* 				create the bulletin board		*/
  /* ------------------------------------------------------------------ */
  return (obj);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Function name: 	PaintColorbarWidget

   Description:		draw the contents of widget

   Date of creation:	16-10-92

   input arguments:	obj	object to redraw

   output:		None

   function return:	void

   Modifications:
   added a second paint procedure. This gives me two color scales

 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void PaintColorbarWidget (SUIT_object obj)
{
  int states, state;
  double x_step;
  GP_point bottom_left, top_right;


  states = SUIT_getInteger (obj, CSTATES);

  /* ----------------------------------------------------------------	*/

  x_step = 1.0 / (double) (states + 1);

  bottom_left.x = 0.0;
  bottom_left.y = 0.0;
  top_right.x = x_step;
  top_right.y = 1.0;

  /* ----------------------------------------------------------------	*/

  for (state = 0; state <= states; state++)
  {
    set_gray (state);

    GP_fillRectanglePt (bottom_left, top_right);

    bottom_left.x = top_right.x;
    top_right.x += x_step;
  }

}

void PaintColorbarWidget2 (SUIT_object obj)
{
  int states, state;
  double x_step;
  GP_point bottom_left, top_right;


  states = SUIT_getInteger (obj, CSTATES);

  /* ----------------------------------------------------------------	*/

  x_step = 1.0 / (double) (states);

  bottom_left.x = 0.0;
  bottom_left.y = 0.0;
  top_right.x = x_step;
  top_right.y = 1.0;

  /* ----------------------------------------------------------------	*/
  /*	progression from black to blue to green to red			*/
  /* ----------------------------------------------------------------	*/

  for (state = 0; state < states; state++)
  {
    set_therm_color (state);

    GP_fillRectanglePt (bottom_left, top_right);

    bottom_left.x = top_right.x;
    top_right.x += x_step;
  }

}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Function name:colorswap

  Description:
  used to swap colours in thermometer colour scale.

  Date of creation:

  input arguments:SUIT hit procedure

  output:

  function return:

  Modifications:

 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void colorswap (SUIT_object obj, SUIT_event ev)
{
  static int hit1 = -1;
  static int hit2 = -1;
  double x;
  int states, counter;
  int value;
  char *tmp;
  FILE *fp;

  states = SUIT_getInteger (obj, CSTATES);

  switch (ev.type)
  {
  case CLICK:
    x = ev.worldLocation.x;

    value = floor (x * (double) states);

    if (hit1 == -1)
    {
      hit1 = value;
    }
    else
    {
      hit2 = value;

      tmp = thermo_colors[hit1];
      thermo_colors[hit1] = thermo_colors[hit2];
      thermo_colors[hit2] = tmp;

      hit2 = hit1 = -1;
      hit2 = value;

      SUIT_paintObject (obj);
    }
    break;

  default:
    fp = fopen ("output", "w+");

    for (counter = 0; counter < states; counter++)
      fprintf (fp, " \"%s\",\n", thermo_colors[counter]);

    fclose (fp);
    SUIT_inform ("written colourmap to file output");
    break;
  }
}
