#include "auto.h"

/************************************************************************/
/*			PRIVATE FUNCTIONS				*/
/************************************************************************/
void PaintMatrixWidget (SUIT_object obj);
void InitMatrixWidget (SUIT_object obj);

int GetMatrixData
  (
    SUIT_object obj,
    SUIT_object * parent,
    int *rows, int *cols, int *max,
    automaton * automata
);

int paint_all_matrix = TRUE;

stack therm_state_list[MAX_THERM_COLORS + 1];
void therm_paint_state_list (automaton automata);
void init_therm_state_list (void);

/************************************************************************/
/* User defined  SUIT objects						*/
/************************************************************************/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	CreateMatrixWidget				*/
/* 									*/
/* Description:		create a new instance of the graphics widget	*/
/*			for displaying Matrix				*/
/* 									*/
/* Date of creation:	10-05-93					*/
/* 									*/
/* input arguments:	name of widget					*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	pointer to new SUIT object			*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

SUIT_object
CreateMatrixWidget (char *name, SUIT_object auto_obj)
{
  SUIT_object graphic_obj, board_obj, label_obj, colour_obj;
  char *graphics_name, *label_name, *colour_name;

  /* ----------------------------------------------------------------	*/
  /* object is a bulletin board with an embedded graphics widget	*/
  /* ----------------------------------------------------------------	*/
  board_obj = SUIT_createBulletinBoard (name);

  /* ----------------------------------------------------------------	*/
  /* 		basic setup of graphics object				*/
  /* ----------------------------------------------------------------	*/
  graphics_name = SUIT_relativeName (board_obj, "graphics");
  graphic_obj = SUIT_createObject (graphics_name, "Matrix class");
  SUIT_addDisplayToObject
    (
      graphic_obj,
      "standard",
      NULL,
      PaintMatrixWidget
    );


  label_name = SUIT_relativeName (board_obj, "text");
  label_obj = SUIT_createLabel (label_name);
  SUIT_setText (label_obj, LABEL, name);

  colour_name = SUIT_relativeName (board_obj, "colorbar");
  colour_obj = CreateColorbarWidget (colour_name);
  SUIT_setEnumString (colour_obj, ACTIVE_DISPLAY, "thermometer");
  SUIT_setInteger (colour_obj, CSTATES, MAX_THERM_COLORS);


  /* ----------------------------------------------------------------	*/
  /* add them to the bulletin board					*/
  /* ----------------------------------------------------------------	*/
  SUIT_addChildToObject (board_obj, graphic_obj);
  SUIT_addChildToObject (board_obj, label_obj);
  SUIT_addChildToObject (board_obj, colour_obj);

  setchildsize (label_obj, 0.0, 0.0, 0.3, 0.89);
  setchildsize (graphic_obj, 0.32, 0.0, 1.0, 0.89);
  setchildsize (colour_obj, 0.00, 0.90, 1.0, 1.0);

  /* ----------------------------------------------------------------	*/
  /* set properties and return the object				*/
  /* ----------------------------------------------------------------	*/
  SUIT_setObject (board_obj, AUTOMATON, auto_obj);
  SUIT_makePropertyTemporary (board_obj, AUTOMATON, OBJECT);
  SUIT_lockProperty (board_obj, AUTOMATON, OBJECT);

  SUIT_setInteger (board_obj, MAX_CUMULATIVE, DEFAULT_MAX_CUMULATIVE);
  SUIT_makePropertyTemporary (board_obj, MAX_CUMULATIVE, OBJECT);

  SUIT_setBoolean (board_obj, REPAINT, FALSE);
  SUIT_makePropertyTemporary (board_obj, REPAINT, OBJECT);

  /* ----------------------------------------------------------------	*/
  /* ----------------------------------------------------------------	*/
  SUIT_setColor (graphic_obj, BACKGROUND_COLOR, GP_defColor ("black"));

  SUIT_setBoolean (board_obj, HAS_BACKGROUND, FALSE);
  SUIT_setBoolean (graphic_obj, HAS_BACKGROUND, FALSE);
  SUIT_setBoolean (graphic_obj, HAS_BORDER, FALSE);


  return (board_obj);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	PaintMatrixWidget				*/
/* 									*/
/* Description:		draw the contents of widget			*/
/* 									*/
/* Date of creation:	11-05-93					*/
/* 									*/
/* input arguments:	obj	object to redraw			*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	void						*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int GetMatrixData
  (
    SUIT_object obj,
    SUIT_object * parent,
    int *rows, int *cols, int *max,
    automaton * automata
)
{
  int changed;
  SUIT_object auto_obj, auto_parent_obj;

  *parent = SUIT_getParent (obj);
  changed = SUIT_getBoolean (*parent, REPAINT);
  if (changed || paint_all_matrix)
  {
    *max = SUIT_getInteger (*parent, MAX_CUMULATIVE);

    auto_parent_obj = SUIT_getObject (*parent, AUTOMATON);
    auto_obj = SUIT_getObject (auto_parent_obj, AUTOMATON);

    *automata = SUITx_getVoidPtr (auto_obj, AUTOMATON);
    *rows = SUIT_getInteger (auto_obj, ROWS);
    *cols = SUIT_getInteger (auto_obj, COLUMNS);
  }
  return (changed);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void PaintMatrixWidget (SUIT_object obj)
{

  int changed;
  SUIT_object parent;
  automaton automata;
  int row, col;
  int item, max, cumulative;
  int cumulative_changed;

  /* ------------------------------------------------------------------	*/
  /* get the information needed for a repaint				*/
  /* ------------------------------------------------------------------	*/
  changed = GetMatrixData (obj, &parent, &rows, &cols, &max, &automata);

  /* ------------------------------------------------------------------	*/
  /* check whether a repaint has been called by the user		*/
  /* ------------------------------------------------------------------	*/
  if (changed && analysis)
  {
    /* ----------------------------------------------------------------	*/
    /* initialise							*/
    /* ----------------------------------------------------------------	*/
    init_therm_state_list ();

    /* ----------------------------------------------------------------	*/
    /* PASS 1 sort items of same colour before display			*/
    /* ----------------------------------------------------------------	*/
    item = 0;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	cumulative = automata[item].cumulative;
	cumulative_changed = automata[item].cumulative_changed;
	if (cumulative_changed) 
	{
	  if (automata[item].unchanged > max)
	  {
	    automata[item].cumulative--;
	    automata[item].unchanged = max;
	    cumulative--;
	  }
	  stack_push( &therm_state_list[cumulative],item);
	  automata[item].cumulative_changed= FALSE;
	}
	item++;
      }

    /* ----------------------------------------------------------------	*/
    /* pass 2 display items						*/
    /* ----------------------------------------------------------------	*/
    therm_paint_state_list(automata);

    /* ----------------------------------------------------------------	*/
    /* repaint no longer needed						*/
    /* ----------------------------------------------------------------	*/
    SUIT_setBoolean (parent, REPAINT, FALSE);
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void InitMatrixWidget (SUIT_object obj)
{
  SUIT_object parent;
  int rows, cols, row, col, item, max;
  automaton automata;

  if (obj)
  {
    GetMatrixData (obj, &parent, &rows, &cols, &max, &automata);

    if (automata)
    {
      item = 0;
      for (row = 0; row < rows; row++)
	for (col = 0; col < cols; col++)
	{
	  automata[item].cumulative = 0;
	  automata[item].cumulative_changed = TRUE;
	  automata[item].unchanged = 0;
	  item++;
	}
    }
    else
    {
     printf("No automata widget to clear\n");
    }


    SUIT_setBoolean (obj, HAS_BACKGROUND, TRUE);
    SUIT_setBoolean (obj, REPAINT, TRUE);
    SUIT_paintObject (obj);
    SUIT_setBoolean (obj, HAS_BACKGROUND, FALSE);
  }
}




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:
*
* Description:
*
* Date of creation:
*
* input arguments:
*
* output:
*
* function return:
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void init_therm_state_list ()
{
  int item;

  for (item = 0; item <= MAX_THERM_COLORS; item++)
    if (therm_state_list[item])
      stack_destroy (&(therm_state_list[item]));
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:
*
* Description:
*
* Date of creation:
*
* input arguments:
*
* output:
*
* function return:
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void therm_paint_state_list (automaton automata)
{
  stack disp_stack;
  int row, col, colour;
  double x, y, x_step, y_step;

  y_step = 1.0 / (double) rows;
  x_step = 1.0 / (double) cols;

  for (colour = 0; colour <= MAX_COLORS; colour++)
  {
    disp_stack = therm_state_list[colour];
    if (disp_stack)
    {
      set_therm_color (colour);
      while (disp_stack)
      {
	get_rowCol (disp_stack->data, &row, &col);

	x = col * x_step;
	y = row * y_step;
	GP_fillRectangleCoord (x, y, x + x_step, y + y_step);

	disp_stack = stack_next (disp_stack);
      }
    }
  }
}



/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:
*
* Description:
*
* Date of creation:
*
* input arguments:
*
* output:
*
* function return:
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void resetMatrix(SUIT_object obj)
{
  char *graphics_name;
  SUIT_object graphics_obj;

  graphics_name = SUIT_relativeName (obj, "graphics");
  graphics_obj = SUIT_name(graphics_name);
  InitMatrixWidget (graphics_obj);
}
