#include "auto.h"


/************************************************************************/
void check_neighbours (
			automaton automata, int row, int col,
			fn_ptr callback,
			stack * update_stack);

int update_changed (automaton automata,
		    stack * update_stack,
		    stack * display_stack);
int elegant_update (automaton automata, stack * up_stack, stack * display_stack);
int slow_update (automaton automata, stack * up_stack, stack * display_stack);

/************************************************************************/

int active;

/************************************************************************/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:	check_neighbours
*
* Description:	checks a 3x3 area around selected cell
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void check_neighbours (
			automaton automata, int row, int col,
			fn_ptr callback,
			stack * update_stack)
{

  int my_row, my_col, item;

  for (my_row = row - 1; my_row <= row + 1; my_row++)
    for (my_col = col - 1; my_col <= col + 1; my_col++)
    {
      item = get_item (my_row, my_col);

      if (!automata[item].checked)
      {
	callback (automata, my_row, my_col, item);
	if (automata[item].state != automata[item].tmp)
	{
	  automata[item].changed = TRUE;
	  stack_push (update_stack, item);
	}
	automata[item].checked = TRUE;
      }
    }
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Function name:	step_method
*
* Description:		does one execution of the selected rule on the
* 			automata
*
* Modifications:
*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int step_method (
		  fn_ptr callback, automaton automata,
		  int rows, int cols,
		  stack * display_stack, stack * update_stack)

{
  int row, col, item;
  int retval;

  /* ------------------------------------------------------------------	*/
  /* The update_stack is generated by the rule and is not valid yet	*/
  /* ------------------------------------------------------------------	*/
  stack_destroy (update_stack);

  /* ------------------------------------------------------------------	*/
  /* check neighbours of all the cells that have a state		*/
  /* ------------------------------------------------------------------	*/
  if (*display_stack)		/********ELEGANT*********/
  {
    while (*display_stack)
    {
      item = (*display_stack)->data;
      stack_pop (display_stack);

      get_rowCol (item, &row, &col);
      check_neighbours (automata, row, col, callback, update_stack);
    }
  }
  else				/******brute force*******/
  {
    item = 0;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	if (automata[item].state > 0)
	  check_neighbours (automata, row, col, callback, update_stack);
	item++;
      }
  }

  /* ------------------------------------------------------------------	*/
  /* update the cells that have changed					*/
  /* ------------------------------------------------------------------	*/
  retval = update_changed (automata, update_stack, display_stack);



  return (retval);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Function name:		update_changed

 Description:		checks which cells have changed and updates the
 			entries in the automaton.

 Modifications:

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int update_changed (automaton automata,
		    stack * update_stack,
		    stack * display_stack)
{
  int changed;
  active = 0;


  stack_destroy (display_stack);

  if (*update_stack)
    changed = elegant_update (automata, update_stack, display_stack);
  else
    changed = slow_update (automata, update_stack, display_stack);

  return(changed);
}


int elegant_update (automaton automata, stack * up_stack, stack * display_stack)
{
  int item, row, col;
  int changed = 0;
  stack update_stack;


  /* ------------------------------------------------------------------	*/
  /* go through all the items that have changed, 			*/
  /* push items onto display stack now and add others later		*/
  /* ------------------------------------------------------------------	*/
  update_stack = *up_stack;

  while (update_stack)
  {
    item = update_stack->data;

    automata[item].state = automata[item].tmp;
    automata[item].cumulative++;
    automata[item].cumulative_changed = TRUE;
    automata[item].unchanged = 0;

    changed++;
    if (automata[item].state > 0)
    {
      stack_push (display_stack, item);
      active++;
    }
    update_stack = stack_next (update_stack);
  }

  /* ------------------------------------------------------------------	*/
  /* and zip along the array to create a display_stack			*/
  /* ------------------------------------------------------------------	*/
  item = 0;
  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
    {
	if (automata[item].state > 0)
	{
	  stack_push (display_stack, item);
	  active++;
	}
	else
	  automata[item].unchanged++;
	automata[item].checked = FALSE;

      item++;
    }

  return (changed);
}


int slow_update (automaton automata, stack * up_stack, stack * display_stack)
{
  int row, col, item;
  int changed = 0;

  /* ------------------------------------------------------------------	*/
  /* ------------------------------------------------------------------	*/
  item = 0;
  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
    {

      if (automata[item].state > 0)
	active++;

      if (automata[item].changed)
      {
	automata[item].state = automata[item].tmp;
	changed++;
	automata[item].cumulative++;
	automata[item].cumulative_changed = TRUE;
	automata[item].unchanged = 0;
      }
      else
      {
	automata[item].unchanged++;
      }

      automata[item].checked = FALSE;
      item++;
    }

  return (changed);
}
