#include "auto.h"

/************************************************************************/
/*			PRIVATE FUNCTIONS				*/
/************************************************************************/
void PaintLineGraphWidget (SUIT_object obj);
void freelist (datalist * list);


/************************************************************************/
/* User defined  SUIT objects						*/
/************************************************************************/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	CreateLineGraphWidget				*/
/* 									*/
/* Description:		create a new instance of the graphics widget	*/
/*			for displaying LineGraph			*/
/* 									*/
/* Date of creation:	10-05-93					*/
/* 									*/
/* input arguments:	name of widget					*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	pointer to new SUIT object			*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

SUIT_object
CreateLineGraphWidget (char *name, void *data)
{
  int states;
  SUIT_object graphic_obj, board_obj, label_obj;
  char *graphics_name, *label_name;

  /* ----------------------------------------------------------------	*/
  /* object is a bulletin board with an embedded graphics widget	*/
  /* ----------------------------------------------------------------	*/
  states = DEFAULT_STATES;

  board_obj = SUIT_createBulletinBoard (name);

  /* ----------------------------------------------------------------	*/
  /* 		basic setup of graphics object				*/
  /* ----------------------------------------------------------------	*/
  graphics_name = SUIT_relativeName (board_obj, "graphics");
  graphic_obj = SUIT_createObject (graphics_name, "LineGraph class");
  SUIT_addDisplayToObject
    (
      graphic_obj,
      "standard",
      NULL,
      PaintLineGraphWidget
    );

  label_name = SUIT_relativeName (board_obj, "text");
  label_obj = SUIT_createLabel (label_name);
  SUIT_setText (label_obj, LABEL, name);


  /* ----------------------------------------------------------------	*/
  /* add them to the bulletin board					*/
  /* ----------------------------------------------------------------	*/
  SUIT_addChildToObject (board_obj, graphic_obj);
  SUIT_addChildToObject (board_obj, label_obj);

  setchildsize (graphic_obj, 0.32, 0.0, 1.0, 1.0);
  setchildsize (label_obj, 0.0, 0.0, 0.3, 1.0);

  /* ----------------------------------------------------------------	*/
  /* set properties and return the object				*/
  /* ----------------------------------------------------------------	*/
  SUITx_setVoidPtr (board_obj, DATA_PTR, data);
  SUIT_makePropertyTemporary (board_obj, DATA_PTR, OBJECT);
  SUIT_lockProperty (board_obj, DATA_PTR, OBJECT);

  SUIT_setBoolean (board_obj, HAS_NEW_DATA, FALSE);
  SUIT_makePropertyTemporary (board_obj, HAS_NEW_DATA, OBJECT);
  SUIT_lockProperty (board_obj, HAS_NEW_DATA, OBJECT);

  SUIT_setInteger (board_obj, NEW_DATA, 0);
  SUIT_makePropertyTemporary (board_obj, NEW_DATA, OBJECT);
  SUIT_lockProperty (board_obj, NEW_DATA, OBJECT);

  SUIT_setInteger (board_obj, HOW_MANY_POINTS, 0);
  SUIT_makePropertyTemporary (board_obj, HOW_MANY_POINTS, OBJECT);
  SUIT_lockProperty (board_obj, HOW_MANY_POINTS, OBJECT);

  SUIT_setInteger (board_obj, MAXIMUM, 0);
  SUIT_makePropertyTemporary (board_obj, MAXIMUM, OBJECT);
  SUIT_lockProperty (board_obj, MAXIMUM, OBJECT);

  /* ----------------------------------------------------------------	*/
  /* ----------------------------------------------------------------	*/
  SUIT_setBoolean (board_obj, HAS_BACKGROUND, FALSE);
  SUIT_setBoolean (graphic_obj, HAS_BACKGROUND, TRUE);
  SUIT_setBoolean (graphic_obj, HAS_BORDER, FALSE);

  SUIT_setColor (graphic_obj, BACKGROUND_COLOR, GP_defColor ("black"));
  SUIT_setColor (graphic_obj, FOREGROUND_COLOR, GP_defColor ("white"));

  return (board_obj);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name: 	PaintLineGraphWidget				*/
/* 									*/
/* Description:		draw the contents of widget			*/
/* 									*/
/* Date of creation:	11-05-93					*/
/* 									*/
/* input arguments:	obj	object to redraw			*/
/* 									*/
/* output:		None						*/
/* 									*/
/* function return:	void						*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void
  PaintLineGraphWidget (SUIT_object obj)
{
  SUIT_object parent;
  int hasnewdata, newdata, howmany, max;
  double separation;
  double scale = 1.0;
  datalist *point_data = NULL;
  datalist *new_point_data;

  /* ----------------------------------------------------------------	*/
  /* get the properties from the parent widget first			*/
  /* ----------------------------------------------------------------	*/
  parent = SUIT_getParent (obj);
  hasnewdata = SUIT_getBoolean (parent, HAS_NEW_DATA);
  howmany = SUIT_getInteger (parent, HOW_MANY_POINTS);
  if (howmany > 0)
    point_data = (datalist *) SUITx_getVoidPtr (parent, DATA_PTR);
  max = SUIT_getInteger (parent, MAXIMUM);

  /* ----------------------------------------------------------------	*/
  /* if there is new data, update the properties			*/
  /* ----------------------------------------------------------------	*/
  if (hasnewdata)
  {

    newdata = SUIT_getInteger (parent, NEW_DATA);
    SUIT_setBoolean (parent, HAS_NEW_DATA, FALSE);

    howmany++;
    SUIT_setInteger (parent, HOW_MANY_POINTS, howmany);

    new_point_data = (datalist *) malloc (sizeof (datalist));
    new_point_data->data = newdata;
    new_point_data->next = (datalistptr) point_data;
    SUITx_setVoidPtr (parent, DATA_PTR, (void *) new_point_data);

    if (newdata > max)
    {
      max = newdata;
      SUIT_setInteger (parent, MAXIMUM, max);
    }

    point_data = new_point_data;
  }

  /* ----------------------------------------------------------------	*/
  /* and readraw, but only if analysis switched on			*/
  /* ----------------------------------------------------------------	*/
  if ((howmany > 0) && analysis)
  {
    double last_x, last_y;
    double x, y, dy;
    int count = 0;

    separation = 1.0 / ((double) howmany);
    if (max > 0)
      scale = 1.0 / ((double) max);

    last_x = last_y = 0.0;
    x = 1.0;

    while (howmany > 0)
    {
      y = scale * point_data->data;
      dy = y - last_y;
      if (dy < 0.0)
	dy = -dy;

      if (((separation > .01) || (dy > 0.1)) && (count > 0))
	GP_lineCoord (last_x, last_y, x, y);
      else
	GP_pointCoord (x, y);

      last_x = x;
      last_y = y;
      point_data = (datalist *) point_data->next;
      howmany--;
      x -= separation;
      count++;
    }
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/*									*/
/* Description:								*/
/*									*/
/* Date of creation:							*/
/*									*/
/* input arguments:							*/
/*									*/
/* output:								*/
/*									*/
/* function return:							*/
/*									*/
/* Modifications:							*/
/*									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
void resetPerfmeter (SUIT_object obj)
{
  datalist *point_data = NULL;

  if (obj)
  {
    point_data = (datalist *) SUITx_getVoidPtr (obj, DATA_PTR);
    freelist ((datalist *) point_data);

    SUITx_setVoidPtr (obj, DATA_PTR, NULL);
    SUIT_setInteger (obj, HOW_MANY_POINTS, 0);
    SUIT_setInteger (obj, MAXIMUM, 0);
  }
}

void freelist (datalist * list)
{
  if (list)
  {
    freelist ((datalist *) list->next);
    free (list);
  }
}
